import {
  Controller,
  Get,
  Delete,
  Patch,
  Put,
  Body,
  UseInterceptors,
  BadRequestException,
  Query,
} from '@nestjs/common'
import { MeService } from './me.service'
import { ApiTags, ApiOperation, ApiUnauthorizedResponse } from '@nestjs/swagger'
import { User } from 'src/common/decorators/user.decorator'
import { NeedAuth } from 'src/common/decorators/need-auth.decorator'
import { PasswordInterceptor } from 'src/common/interceptors/password.interceptor'
import { PrismaService } from 'nestjs-prisma'
import { UserEntity } from './entities/user.entity'
import { DeleteUserDto } from './dto/delete-user.dto'
import { ChangePassword } from './dto/change-password.dto'
import { UpdateUserDto } from './dto/update-user.dto'
import { TokenRefreshPayload } from './dto/token.dto'
import { ChangeEmailDto } from './dto/change-email.dto'

@Controller('api/users/me')
@ApiTags('Me')
export class MeController {
  constructor(
    private readonly meService: MeService,
    private readonly prismaService: PrismaService,
  ) {}

  @Get()
  @ApiOperation({ summary: '获取用户信息' })
  @UseInterceptors(PasswordInterceptor)
  @NeedAuth()
  async getUserInfo(@User('userId') userId: string): Promise<UserEntity> {
    return this.prismaService.user.findUniqueOrThrow({
      where: { id: userId },
    })
  }

  @Patch()
  @ApiOperation({ summary: '修改用户信息（用户名等）' })
  @UseInterceptors(PasswordInterceptor)
  @NeedAuth()
  async updateUserInfo(
    @Body() payload: UpdateUserDto,
    @User('userId') userId: string,
  ): Promise<UserEntity> {
    if (Object.keys(payload).length === 0) {
      throw new BadRequestException()
    }
    return this.prismaService.user.update({
      where: { id: userId },
      data: payload,
    })
  }

  @Delete()
  @NeedAuth()
  @ApiOperation({ summary: '删除用户' })
  async deleteUser(
    @Body() userData: DeleteUserDto,
    @User('userId') userId: string,
  ) {
    return this.meService.deleteUser(userData, userId)
  }

  @Patch('password')
  @NeedAuth()
  @ApiOperation({ summary: '修改密码' })
  @UseInterceptors(PasswordInterceptor)
  async changePassword(
    @Body() payload: ChangePassword,
    @User('userId') userId: string,
  ) {
    return this.meService.changePassword(payload, userId)
  }

  @Patch('email')
  @NeedAuth()
  @ApiOperation({ summary: '修改邮箱' })
  @UseInterceptors(PasswordInterceptor)
  async changeEmail(
    @Query() query,
    @Body() payload: ChangeEmailDto,
    @User('userId') userId: string,
  ): Promise<UserEntity> {
    console.log(query)
    return this.meService.changeEmail(payload, userId)
  }

  @Put('token')
  @ApiOperation({ summary: '刷新token' })
  @ApiUnauthorizedResponse({ description: 'Unauthorized' })
  async updateAccessToken(@Body() payload: TokenRefreshPayload) {
    return this.meService.updateAccessToken(payload.refreshToken)
  }
}
